/**
 * @file 通用工具类
 * @author r2space@gmail.com
 */

"use strict";

var fs          = require("fs")
  , ejs         = require("ejs")
  , os          = require("os")
  , _           = require("underscore");

/**
 * 简单生成随机4位字符串
 */
exports.randomGUID4 = function() {
  return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
};

/**
 * 简单生成随机8位字符串, 会有重复数据生成
 * GUID : Global Unique Identifier
 */
exports.randomGUID8 = function() {
  return exports.randomGUID4() + exports.randomGUID4();
};

/**
 * 读取模板文件，带入参数，生成结果文件，如果没有指定结果文件，则返回解析后的字符串
 * @param {String} templateFile ejs模板文件
 * @param {Object} parameters 模板文件参数对象
 * @param {String} resultFile 结果文件，如果没有指定则以字符串的形式返回解析的内容
 * @returns {String}
 */
exports.ejsParser = function(templateFile, parameters, resultFile) {

  // 读取模板文件
  var template = fs.readFileSync(templateFile, "utf8");

  // 转换模板文件
  var result = ejs.render(template, parameters);

  // 没有指定输出文件，则返回字符串
  if (!resultFile) {
    return result;
  }

  // 输出文件
  fs.writeFileSync(resultFile, result);
  return undefined;
};

/**
 * 判断客户端是否是浏览器
 * @param {Object} req 请求
 * @returns {*}
 */
exports.isBrowser = function(req) {
  var userAgent = req.headers["user-agent"].toLowerCase();
  return userAgent.match(/mozilla.*/i);
};

/**
 * 返回客户端类型
 * @param {Object} req 请求
 * @returns {*} 浏览器返回‘mozilla‘，ios应用返回’app名称‘
 */
exports.clientType = function(req) {
  var userAgent = req.headers["user-agent"].toLowerCase();
  return userAgent.split("/")[0];
};

/**
 * 获取AP服务器IP地址的数组，获取的IP地址放到global对象中缓存
 * @returns 返回IP地址
 */
exports.ip = function() {

  if (global.addresses) {
    return global.addresses;
  }

  var interfaces = os.networkInterfaces()
    , addresses = [];

  _.each(interfaces, function(item) {
    _.each(item, function(address) {
      if (address.family === "IPv4" && !address.internal) {
        addresses.push(address.address);
      }
    });
  });

  global.addresses = addresses;
  return global.addresses;
};

/**
 * 获取应用程序情报
 * @returns {Object} 应用程序版本信息等
 */
exports.applicationInfo = function() {

  var app = require(process.cwd() + "/package.json");
  return {
      version: app.version
    , host: os.hostname()
    , application: app.name
    , time: new Date()
    };
};

/**
 * 判断模块是否可以加载
 *  TODO: cwd方法的目录依赖，会因为启动方式，启动目录不同而不准确
 *  TODO: 是否用代码文件存在来判断更加合理？而不是用一场捕获
 * @param module 模块名称
 * @returns {String} 路径
 */
exports.resolve = function(module) {
  try {
    return require(process.cwd() + module);
  } catch(e) {
    return undefined;
  }
};


/**
 * 加载给定的字符串，与eval类似
 * @param {String} src
 * @param {String} filename
 * @returns {Object}
 */
exports.requireFromString = function(src, filename) {
  var Module = module.constructor;
  var m = new Module();
  m._compile(src, filename);
  return m.exports;
};
