/**
 * @file controller的共通类
 * @author r2space@gmail.com
 */

"use strict";

var sync      = require("async")
  , mongo     = require("mongoose")
  , _         = require("underscore")
  , Schema    = mongo.Schema
  , errors    = require("../error")
  , constant  = require("../constant")
  , model     = require("./model");

/**
 * 获取单例schema对象（mongoose要求schema只能被定义一次）
 * @type {{}}
 */
var schemas = {};
function schema(name, define) {
  if (schemas[name]) {
    return schemas[name];
  }

  schemas[name] = new Schema(define);
  return schemas[name];
};

function fields(define) {
  var result = _.keys(define);

  // remove common field
  delete result.valid;
  delete result.createAt;
  delete result.createBy;
  delete result.updateAt;
  delete result.updateBy;

  return result;
}

/**
 * 构造函数
 * @type {Controller}
 */
var Controller = module.exports = function Controller(handler, name, define) {

  this.schema   = schema(name, define);
  this.model    = new model(handler.code, name, this.schema);
  this.fields   = fields(define);
  this.handler  = handler;
  this.uid      = handler.uid;
  this.params   = handler.params;
  this.id       = handler.params.id;
};

Controller.prototype.add = function(callback) {

  var obj = _.pick(this.params.data || this.params, this.fields);
  obj.createAt = new Date();
  obj.updateAt = obj.createAt;
  obj.createBy = this.uid;
  obj.updateBy = this.uid
  obj.valid = constant.VALID;

  this.model.add(obj, function(err, result) {
    if (err) {
      return callback(new errors.db.Add());
    }
    return callback(err, result);
  });
};

Controller.prototype.remove = function(callback) {
  this.model.remove(this.id, { updateAt: new Date(), updateBy: this.handler.uid }, function(err, result) {
    if (err) {
      return callback(new errors.db.Remove());
    }
    return callback(err, result);
  });
};

Controller.prototype.update = function(callback) {

  var obj = _.pick(this.params.data || this.params, this.fields);
  obj.updateAt = new Date();
  obj.updateBy = this.uid
  obj.valid = constant.VALID;

  this.model.update(this.id, obj, function(err, result) {
    if (err) {
      return callback(new errors.db.Update());
    }
    if (result) {
      return callback(err, result);
    } else {
      return callback(new errors.db.NotExist());
    }
  });
};

Controller.prototype.get = function(callback) {
  this.model.get(this.id, function (err, result) {
    if (err) {
      return callback(new errors.db.Find());
    }
    return callback(err, result);
  });
};

Controller.prototype.getOne = function(callback) {
  this.model.getOne(this.params.condition, function (err, result) {
    if (err) {
      return callback(new errors.db.Find());
    }
    return callback(err, result);
  });
};

Controller.prototype.getList = function(callback) {
  var self      = this
    , start     = this.params.start
    , limit     = this.params.limit
    , condition = this.params.condition
    , order     = this.params.order
    , select    = this.params.select;

  self.model.total(condition, function (err, count) {
    if (err) {
      callback(new errors.db.Find());
      return;
    }

    self.model.getList(condition, start, limit, order, select, function (err, result) {
      if (err) {
        callback(new errors.db.Find());
      } else {
        callback(err, { totalItems: count, items: result });
      }
    });
  });
};

Controller.prototype.distinct = function(callback) {
  this.model.distinct(this.params.field, this.params.condition, function (err, result) {
    if (err) {
      return callback(new errors.db.Find());
    }
    return callback(err, result);
  });
};

Controller.prototype.total = function(callback) {
  this.model.total(this.params.condition, function (err, result) {
    if (err) {
      return callback(new errors.db.Find());
    }
    return callback(err, result);
  });
};

Controller.prototype.waterfall = function(functions, final) {
  sync.waterfall(functions, final);
};

Controller.prototype.schema = function() {
  return this.model.schema();
};

Controller.prototype.increment = function(callback) {

  console.log(this.params);
  this.model.increment(this.params.condition, this.params.select, function (err, result) {
    if (err) {
      return callback(new errors.db.Update());
    }
    return callback(err, result);
  });
};
